@@ -38,6 +38,8 @@
 #define HELPID_INACTIVE		1004
 #define HELPID_DEBUG            1005
 #define HELPID_VERSION		1006
+#define HELPID_NOCHECK		1007
+#define HELPID_ABSTRACT		1008
 
 #define LEVEL_BASIC		(1<<0)
 #define LEVEL_INACTIVE		(1<<1)
@@ -45,6 +47,9 @@
 
 int quiet = 0;
 int debugflag = 0;
+int no_check = 0;
+int smixer_level = 0;
+struct snd_mixer_selem_regopt smixer_options;
 char card[64] = "default";
 
 static void error(const char *fmt,...)
@@ -66,9 +71,11 @@ static int help(void)
 	printf("  -c,--card N     select the card\n");
 	printf("  -D,--device N   select the device, default '%s'\n", card);
 	printf("  -d,--debug      debug mode\n");
+	printf("  -n,--nocheck    do not perform range checking\n");
 	printf("  -v,--version    print version of this program\n");
 	printf("  -q,--quiet      be quiet\n");
 	printf("  -i,--inactive   show also inactive controls\n");
+	printf("  -a,--abstract L select abstraction level (none or basic)\n");
 	printf("\nAvailable commands:\n");
 	printf("  scontrols       show all mixer simple controls\n");
 	printf("  scontents	  show contents of all mixer simple controls (default command)\n");
@@ -119,7 +126,7 @@ static int info(void)
 		snd_mixer_close(mhandle);
 		return err;
 	}
-	if ((err = snd_mixer_selem_register(mhandle, NULL, NULL)) < 0) {
+	if ((err = snd_mixer_selem_register(mhandle, smixer_level > 0 ? &smixer_options : NULL, NULL)) < 0) {
 		error("Mixer register error: %s", snd_strerror(err));
 		snd_mixer_close(mhandle);
 		return err;
@@ -161,6 +168,8 @@ static const char *control_access(snd_ctl_elem_info_t *info)
 
 static int check_range(int val, int min, int max)
 {
+	if (no_check)
+		return val;
 	if (val < min)
 		return min;
 	if (val > max)
@@ -767,7 +776,7 @@ static int selems(int level)
 		snd_mixer_close(handle);
 		return err;
 	}
-	if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0) {
+	if ((err = snd_mixer_selem_register(handle, smixer_level > 0 ? &smixer_options : NULL, NULL)) < 0) {
 		error("Mixer register error: %s", snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
@@ -1138,7 +1147,7 @@ static int sset(unsigned int argc, char *argv[], int roflag)
 		snd_mixer_close(handle);
 		return err;
 	}
-	if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0) {
+	if ((err = snd_mixer_selem_register(handle, smixer_level > 0 ? &smixer_options : NULL, NULL)) < 0) {
 		error("Mixer register error: %s", snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
@@ -1355,6 +1364,7 @@ static int events(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 		}
 	}
 	snd_hctl_close(handle);
+	return 0;
 }
 
 static void sevents_value(snd_mixer_selem_id_t *sid)
@@ -1419,7 +1429,7 @@ static int sevents(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 		snd_mixer_close(handle);
 		return err;
 	}
-	if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0) {
+	if ((err = snd_mixer_selem_register(handle, smixer_level > 0 ? &smixer_options : NULL, NULL)) < 0) {
 		error("Mixer register error: %s", snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
@@ -1443,6 +1453,7 @@ static int sevents(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 		}
 	}
 	snd_mixer_close(handle);
+	return 0;
 }
 
 int main(int argc, char *argv[])
@@ -1456,7 +1467,9 @@ int main(int argc, char *argv[])
 		{"quiet", 0, NULL, HELPID_QUIET},
 		{"inactive", 0, NULL, HELPID_INACTIVE},
 		{"debug", 0, NULL, HELPID_DEBUG},
+		{"nocheck", 0, NULL, HELPID_NOCHECK},
 		{"version", 0, NULL, HELPID_VERSION},
+		{"abstract", 1, NULL, HELPID_ABSTRACT},
 		{NULL, 0, NULL, 0},
 	};
 
@@ -1464,13 +1477,13 @@ int main(int argc, char *argv[])
 	while (1) {
 		int c;
 
-		if ((c = getopt_long(argc, argv, "hc:D:qidv", long_option, NULL)) < 0)
+		if ((c = getopt_long(argc, argv, "hc:D:qidnva:", long_option, NULL)) < 0)
 			break;
 		switch (c) {
 		case 'h':
 		case HELPID_HELP:
-			morehelp++;
-			break;
+			help();
+			return 0;
 		case 'c':
 		case HELPID_CARD:
 			{
@@ -1501,10 +1514,28 @@ int main(int argc, char *argv[])
 		case HELPID_DEBUG:
 			debugflag = 1;
 			break;
+		case 'n':
+		case HELPID_NOCHECK:
+			no_check = 1;
+			break;
 		case 'v':
 		case HELPID_VERSION:
 			printf("amixer version " SND_UTIL_VERSION_STR "\n");
 			return 1;
+		case 'a':
+		case HELPID_ABSTRACT:
+			smixer_level = 1;
+			memset(&smixer_options, 0, sizeof(smixer_options));
+			smixer_options.ver = 1;
+			if (!strcmp(optarg, "none"))
+				smixer_options.abstract = SND_MIXER_SABSTRACT_NONE;
+			else if (!strcmp(optarg, "basic"))
+				smixer_options.abstract = SND_MIXER_SABSTRACT_BASIC;
+			else {
+				fprintf(stderr, "Select correct abstraction level (none or basic)...\n");
+				morehelp++;
+			}
+			break;
 		default:
 			fprintf(stderr, "\07Invalid switch or option needs an argument.\n");
 			morehelp++;
