@@ -51,6 +51,7 @@
 #define LLONG_MAX    9223372036854775807LL
 #endif
 
+#define DEFAULT_FORMAT		SND_PCM_FORMAT_U8
 #define DEFAULT_SPEED 		8000
 
 #define FORMAT_DEFAULT		-1
@@ -185,6 +186,24 @@ static void usage(char *command)
 	fprintf(stderr, "-f dat (16 bit little endian, 48000, stereo)\n");
 }
 
+static void names_list(void)
+{
+	int err;
+	snd_devname_t *list, *item;
+
+	err = snd_names_list("pcm", &list);
+	if (err < 0) {
+		error("snd_names_list error: %s", snd_strerror(err));
+		return;
+	}
+	item = list;
+	while (item) {
+		printf("%s [%s]\n", item->name, item->comment);
+		item = item->next;
+	}
+	snd_names_list_free(list);
+}
+
 static void device_list(void)
 {
 	snd_ctl_t *handle;
@@ -199,6 +218,7 @@ static void device_list(void)
 		error("no soundcards found...");
 		return;
 	}
+	fprintf(stderr, "**** List of %s Hardware Devices ****\n", snd_pcm_stream_name(stream));
 	while (card >= 0) {
 		char name[32];
 		sprintf(name, "hw:%d", card);
@@ -211,7 +231,6 @@ static void device_list(void)
 			snd_ctl_close(handle);
 			goto next_card;
 		}
-		fprintf(stderr, "**** List of %s Hardware Devices ****\n", snd_pcm_stream_name(stream));
 		dev = -1;
 		while (1) {
 			unsigned int count;
@@ -304,10 +323,11 @@ enum {
 int main(int argc, char *argv[])
 {
 	int option_index;
-	char *short_options = "hlLD:qt:c:f:r:d:s:MNF:A:X:R:T:B:vIPC";
+	char *short_options = "hnlLD:qt:c:f:r:d:s:MNF:A:R:T:B:vIPC";
 	static struct option long_options[] = {
 		{"help", 0, 0, 'h'},
 		{"version", 0, 0, OPT_VERSION},
+		{"list-devnames", 0, 0, 'n'},
 		{"list-devices", 0, 0, 'l'},
 		{"list-pcms", 0, 0, 'L'},
 		{"device", 1, 0, 'D'},
@@ -317,7 +337,7 @@ int main(int argc, char *argv[])
 		{"format", 1, 0, 'f'},
 		{"rate", 1, 0, 'r'},
 		{"duration", 1, 0 ,'d'},
-		{"sleep-min", 0, 0, 's'},
+		{"sleep-min", 1, 0, 's'},
 		{"mmap", 0, 0, 'M'},
 		{"nonblock", 0, 0, 'N'},
 		{"period-time", 1, 0, 'F'},
@@ -335,7 +355,7 @@ int main(int argc, char *argv[])
 	};
 	char *pcm_name = "default";
 	int tmp, err, c;
-	int do_device_list = 0, do_pcm_list = 0;
+	int do_names_list = 0, do_device_list = 0, do_pcm_list = 0;
 	snd_pcm_info_t *info;
 
 	snd_pcm_info_alloca(&info);
@@ -359,7 +379,7 @@ int main(int argc, char *argv[])
 	}
 
 	chunk_size = -1;
-	rhwparams.format = SND_PCM_FORMAT_U8;
+	rhwparams.format = DEFAULT_FORMAT;
 	rhwparams.rate = DEFAULT_SPEED;
 	rhwparams.channels = 1;
 
@@ -371,6 +391,9 @@ int main(int argc, char *argv[])
 		case OPT_VERSION:
 			version();
 			return 0;
+		case 'n':
+			do_names_list = 1;
+			break;
 		case 'l':
 			do_device_list = 1;
 			break;
@@ -491,7 +514,10 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (do_device_list) {
+	if (do_names_list) {
+		names_list();
+		return 0;
+	} else if (do_device_list) {
 		if (do_pcm_list) pcm_list();
 		device_list();
 		snd_config_update_free_global();
@@ -601,12 +627,12 @@ static int test_vocfile(void *buffer)
 {
 	VocHeader *vp = buffer;
 
-	if (strstr(vp->magic, VOC_MAGIC_STRING)) {
-		vocminor = vp->version & 0xFF;
-		vocmajor = vp->version / 256;
-		if (vp->version != (0x1233 - vp->coded_ver))
+	if (!memcmp(vp->magic, VOC_MAGIC_STRING, 20)) {
+		vocminor = LE_SHORT(vp->version) & 0xFF;
+		vocmajor = LE_SHORT(vp->version) / 256;
+		if (LE_SHORT(vp->version) != (0x1233 - LE_SHORT(vp->coded_ver)))
 			return -2;	/* coded version mismatch */
-		return vp->headerlen - sizeof(VocHeader);	/* 0 mostly */
+		return LE_SHORT(vp->headerlen) - sizeof(VocHeader);	/* 0 mostly */
 	}
 	return -1;		/* magic string fail */
 }
@@ -626,12 +652,13 @@ size_t test_wavefile_read(int fd, char *buffer, size_t *size, size_t reqsize, in
 	return *size = reqsize;
 }
 
-#define check_wavefile_space(buffer, size, len, blimit) \
-	if (size + len > blimit) \
-		blimit = size + len; \
-	if ((buffer = realloc(buffer, blimit)) == NULL) { \
-		error("not enough memory"); \
-		exit(EXIT_FAILURE); \
+#define check_wavefile_space(buffer, len, blimit) \
+	if (len > blimit) { \
+		blimit = len; \
+		if ((buffer = realloc(buffer, blimit)) == NULL) { \
+			error("not enough memory"); \
+			exit(EXIT_FAILURE); \
+		} \
 	}
 
 /*
@@ -653,22 +680,23 @@ static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
 	if (h->magic != WAV_RIFF || h->type != WAV_WAVE)
 		return -1;
 	if (size > sizeof(WaveHeader)) {
-		check_wavefile_space(buffer, size, size - sizeof(WaveHeader), blimit);
+		check_wavefile_space(buffer, size - sizeof(WaveHeader), blimit);
 		memcpy(buffer, _buffer + sizeof(WaveHeader), size - sizeof(WaveHeader));
 	}
 	size -= sizeof(WaveHeader);
 	while (1) {
-		check_wavefile_space(buffer, size, sizeof(WaveChunkHeader), blimit);
+		check_wavefile_space(buffer, sizeof(WaveChunkHeader), blimit);
 		test_wavefile_read(fd, buffer, &size, sizeof(WaveChunkHeader), __LINE__);
 		c = (WaveChunkHeader*)buffer;
 		type = c->type;
 		len = LE_INT(c->length);
+		len += len % 2;
 		if (size > sizeof(WaveChunkHeader))
 			memmove(buffer, buffer + sizeof(WaveChunkHeader), size - sizeof(WaveChunkHeader));
 		size -= sizeof(WaveChunkHeader);
 		if (type == WAV_FMT)
 			break;
-		check_wavefile_space(buffer, size, len, blimit);
+		check_wavefile_space(buffer, len, blimit);
 		test_wavefile_read(fd, buffer, &size, len, __LINE__);
 		if (size > len)
 			memmove(buffer, buffer + len, size - len);
@@ -679,7 +707,7 @@ static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
 		error("unknown length of 'fmt ' chunk (read %u, should be %u at least)", len, (u_int)sizeof(WaveFmtBody));
 		exit(EXIT_FAILURE);
 	}
-	check_wavefile_space(buffer, size, len, blimit);
+	check_wavefile_space(buffer, len, blimit);
 	test_wavefile_read(fd, buffer, &size, len, __LINE__);
 	f = (WaveFmtBody*) buffer;
 	if (LE_SHORT(f->format) != WAV_PCM_CODE) {
@@ -693,17 +721,29 @@ static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
 	hwparams.channels = LE_SHORT(f->modus);
 	switch (LE_SHORT(f->bit_p_spl)) {
 	case 8:
+		if (hwparams.format != DEFAULT_FORMAT &&
+		    hwparams.format != SND_PCM_FORMAT_U8)
+			fprintf(stderr, "Warning: format is changed to U8\n");
 		hwparams.format = SND_PCM_FORMAT_U8;
 		break;
 	case 16:
+		if (hwparams.format != DEFAULT_FORMAT &&
+		    hwparams.format != SND_PCM_FORMAT_S16_LE)
+			fprintf(stderr, "Warning: format is changed to S16_LE\n");
 		hwparams.format = SND_PCM_FORMAT_S16_LE;
 		break;
 	case 24:
 		switch (LE_SHORT(f->byte_p_spl) / hwparams.channels) {
 		case 3:
+			if (hwparams.format != DEFAULT_FORMAT &&
+			    hwparams.format != SND_PCM_FORMAT_S24_3LE)
+				fprintf(stderr, "Warning: format is changed to S24_3LE\n");
 			hwparams.format = SND_PCM_FORMAT_S24_3LE;
 			break;
 		case 4:
+			if (hwparams.format != DEFAULT_FORMAT &&
+			    hwparams.format != SND_PCM_FORMAT_S24_LE)
+				fprintf(stderr, "Warning: format is changed to S24_LE\n");
 			hwparams.format = SND_PCM_FORMAT_S24_LE;
 			break;
 		default:
@@ -727,7 +767,7 @@ static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
 	while (1) {
 		u_int type, len;
 
-		check_wavefile_space(buffer, size, sizeof(WaveChunkHeader), blimit);
+		check_wavefile_space(buffer, sizeof(WaveChunkHeader), blimit);
 		test_wavefile_read(fd, buffer, &size, sizeof(WaveChunkHeader), __LINE__);
 		c = (WaveChunkHeader*)buffer;
 		type = c->type;
@@ -736,14 +776,15 @@ static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
 			memmove(buffer, buffer + sizeof(WaveChunkHeader), size - sizeof(WaveChunkHeader));
 		size -= sizeof(WaveChunkHeader);
 		if (type == WAV_DATA) {
-			if (len < pbrec_count)
+			if (len < pbrec_count && len < 0x7ffffffe)
 				pbrec_count = len;
 			if (size > 0)
 				memcpy(_buffer, buffer, size);
 			free(buffer);
 			return size;
 		}
-		check_wavefile_space(buffer, size, len, blimit);
+		len += len % 2;
+		check_wavefile_space(buffer, len, blimit);
 		test_wavefile_read(fd, buffer, &size, len, __LINE__);
 		if (size > len)
 			memmove(buffer, buffer + len, size - len);
@@ -769,12 +810,21 @@ static int test_au(int fd, void *buffer)
 	pbrec_count = BE_INT(ap->data_size);
 	switch (BE_INT(ap->encoding)) {
 	case AU_FMT_ULAW:
+		if (hwparams.format != DEFAULT_FORMAT &&
+		    hwparams.format != SND_PCM_FORMAT_MU_LAW)
+			fprintf(stderr, "Warning: format is changed to MU_LAW\n");
 		hwparams.format = SND_PCM_FORMAT_MU_LAW;
 		break;
 	case AU_FMT_LIN8:
+		if (hwparams.format != DEFAULT_FORMAT &&
+		    hwparams.format != SND_PCM_FORMAT_U8)
+			fprintf(stderr, "Warning: format is changed to U8\n");
 		hwparams.format = SND_PCM_FORMAT_U8;
 		break;
 	case AU_FMT_LIN16:
+		if (hwparams.format != DEFAULT_FORMAT &&
+		    hwparams.format != SND_PCM_FORMAT_S16_BE)
+			fprintf(stderr, "Warning: format is changed to S16_BE\n");
 		hwparams.format = SND_PCM_FORMAT_S16_BE;
 		break;
 	default:
@@ -930,6 +980,7 @@ static void set_params(void)
 
 	err = snd_pcm_sw_params_set_xfer_align(handle, swparams, xfer_align);
 	assert(err >= 0);
+
 	if (snd_pcm_sw_params(handle, swparams) < 0) {
 		error("unable to install sw params:");
 		snd_pcm_sw_params_dump(swparams, log);
@@ -1037,19 +1088,46 @@ static void suspend(void)
 static void compute_max_peak(u_char *data, size_t count)
 {
 	signed int val, max, max_peak = 0, perc;
-	size_t step, ocount = count;
+	size_t ocount = count;
 	
-	while (count-- > 0) {
-		switch (bits_per_sample) {
-		case 8: val = *(signed char *)data ^ snd_pcm_format_silence(hwparams.format); step = 1; break;
-		case 16: val = *(signed short *)data ^ snd_pcm_format_silence_16(hwparams.format); step = 2; break;
-		case 32: val = *(signed int *)data ^ snd_pcm_format_silence_32(hwparams.format); step = 4; break;
-		default: val = 0; step = 1; break;
+	switch (bits_per_sample) {
+	case 8: {
+		signed char *valp = (signed char *)data;
+		signed char mask = snd_pcm_format_silence(hwparams.format);
+		while (count-- > 0) {
+			val = *valp++ ^ mask;
+			val = abs(val);
+			if (max_peak < val)
+				max_peak = val;
 		}
-		data += step;
-		val = abs(val);
-		if (max_peak < val)
-			max_peak = val;
+		break;
+	}
+	case 16: {
+		signed short *valp = (signed short *)data;
+		signed short mask = snd_pcm_format_silence_16(hwparams.format);
+		count /= 2;
+		while (count-- > 0) {
+			val = *valp++ ^ mask;
+			val = abs(val);
+			if (max_peak < val)
+				max_peak = val;
+		}
+		break;
+	}
+	case 32: {
+		signed int *valp = (signed int *)data;
+		signed int mask = snd_pcm_format_silence_32(hwparams.format);
+		count /= 4;
+		while (count-- > 0) {
+			val = *valp++ ^ mask;
+			val = abs(val);
+			if (max_peak < val)
+				max_peak = val;
+		}
+		break;
+	}
+	default:
+		break;
 	}
 	max = 1 << (bits_per_sample-1);
 	if (max <= 0)
@@ -1272,6 +1350,7 @@ static void voc_write_silence(unsigned x)
 		}
 		x -= l;
 	}
+	free(buf);
 }
 
 static void voc_pcm_flush(void)
@@ -1330,7 +1409,7 @@ static void voc_play(int fd, int ofs, char *name)
 			exit(EXIT_FAILURE);
 		}
 	}
-	hwparams.format = SND_PCM_FORMAT_U8;
+	hwparams.format = DEFAULT_FORMAT;
 	hwparams.channels = 1;
 	hwparams.rate = DEFAULT_SPEED;
 	set_params();
@@ -1540,11 +1619,10 @@ static void begin_voc(int fd, size_t cnt)
 	VocVoiceData vd;
 	VocExtBlock eb;
 
-	strncpy(vh.magic, VOC_MAGIC_STRING, 20);
-	vh.magic[19] = 0x1A;
-	vh.headerlen = sizeof(VocHeader);
-	vh.version = VOC_ACTUAL_VERSION;
-	vh.coded_ver = 0x1233 - VOC_ACTUAL_VERSION;
+	memcpy(vh.magic, VOC_MAGIC_STRING, 20);
+	vh.headerlen = LE_SHORT(sizeof(VocHeader));
+	vh.version = LE_SHORT(VOC_ACTUAL_VERSION);
+	vh.coded_ver = LE_SHORT(0x1233 - VOC_ACTUAL_VERSION);
 
 	if (write(fd, &vh, sizeof(VocHeader)) != sizeof(VocHeader)) {
 		error("write error");
@@ -1559,7 +1637,7 @@ static void begin_voc(int fd, size_t cnt)
 			error("write error");
 			exit(EXIT_FAILURE);
 		}
-		eb.tc = (u_short) (65536 - 256000000L / (hwparams.rate << 1));
+		eb.tc = LE_SHORT(65536 - 256000000L / (hwparams.rate << 1));
 		eb.pack = 0;
 		eb.mode = 1;
 		if (write(fd, &eb, sizeof(VocExtBlock)) != sizeof(VocExtBlock)) {
@@ -1721,15 +1799,16 @@ static void end_wave(int fd)
 {				/* only close output */
 	WaveChunkHeader cd;
 	off64_t length_seek;
+	off64_t filelen;
 	u_int rifflen;
 	
 	length_seek = sizeof(WaveHeader) +
 		      sizeof(WaveChunkHeader) +
 		      sizeof(WaveFmtBody);
 	cd.type = WAV_DATA;
-	cd.length = fdcount > 0x7fffffff ? 0x7fffffff : LE_INT(fdcount);
-	rifflen = fdcount + 2*sizeof(WaveChunkHeader) + sizeof(WaveFmtBody) + 4;
-	rifflen = rifflen > 0x7fffffff ? 0x7fffffff : LE_INT(rifflen);
+	cd.length = fdcount > 0x7fffffff ? LE_INT(0x7fffffff) : LE_INT(fdcount);
+	filelen = fdcount + 2*sizeof(WaveChunkHeader) + sizeof(WaveFmtBody) + 4;
+	rifflen = filelen > 0x7fffffff ? LE_INT(0x7fffffff) : LE_INT(filelen);
 	if (lseek64(fd, 4, SEEK_SET) == 4)
 		write(fd, &rifflen, 4);
 	if (lseek64(fd, length_seek, SEEK_SET) == length_seek)
@@ -1834,9 +1913,7 @@ void capture_go(int fd, off64_t count, int rtype, char *name)
 
 	do {
 		for (cur = count; cur > 0; cur -= r) {
-			c = cur;
-			if (c > chunk_bytes)
-				c = chunk_bytes;
+			c = (cur <= chunk_bytes) ? cur : chunk_bytes;
 			c = c * 8 / bits_per_frame;
 			if ((size_t)(r = pcm_read(audiobuf, c)) != c)
 				break;
@@ -1879,7 +1956,7 @@ static void playback(char *name)
 		exit(EXIT_FAILURE);
 	}
 	if (test_au(fd, audiobuf) >= 0) {
-		rhwparams.format = SND_PCM_FORMAT_MU_LAW;
+		rhwparams.format = hwparams.format;
 		pbrec_count = calc_count();
 		playback_go(fd, 0, pbrec_count, FORMAT_AU, name);
 		goto __end;
