@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000-2004 James Courtier-Dutton
+ * Copyright (C) 2005 Nathan Hurst
  *
  * This file is part of the speaker-test tool.
  *
@@ -22,8 +23,17 @@
  *
  * Main program by James Courtier-Dutton (including some source code fragments from the alsa project.)
  * Some cleanup from Daniel Caujolle-Bert <segfault@club-internet.fr>
+ * Pink noise option added Nathan Hurst, 
+ *   based on generator by Phil Burk (pink.c)
  *
- * speaker-test.c,v 1.2 2004/05/16 13:43:29 perex Exp
+ * Changelog:
+ *   0.0.8 Added support for pink noise output.
+ * Changelog:
+ *   0.0.7 Added support for more than 6 channels.
+ * Changelog:
+ *   0.0.6 Added support for different sample formats.
+ *
+ * speaker-test.c,v 1.8 2005/05/03 18:36:08 jcdutton Exp
  */
 
 #include <stdio.h>
@@ -32,11 +42,16 @@
 #include <sched.h>
 #include <errno.h>
 #include <getopt.h>
-#include "aconfig.h"
+#include <inttypes.h>
 
+/* #include "aconfig.h" */
+
+#define ALSA_PCM_NEW_HW_PARAMS_API
+#define ALSA_PCM_NEW_SW_PARAMS_API
 #include <alsa/asoundlib.h>
 #include <sys/time.h>
 #include <math.h>
+#include "pink.h"
 
 static char              *device      = "plughw:0,0";       /* playback device */
 static snd_pcm_format_t   format      = SND_PCM_FORMAT_S16; /* sample format */
@@ -45,7 +60,10 @@ static unsigned int       channels    = 1;	            /* count of channels */
 static unsigned int       speaker     = 0;	            /* count of channels */
 static unsigned int       buffer_time = 500000;	            /* ring buffer length in us */
 static unsigned int       period_time = 100000;	            /* period time in us */
+#define PERIODS 4
 static double             freq        = 440;                /* sinusoidal wave frequency in Hz */
+static int                test_type   = 1;                  /* Test type. 1 = noise, 2 = sine wave */
+static pink_noise_t pink;
 static snd_output_t      *output      = NULL;
 static snd_pcm_uframes_t  buffer_size;
 static snd_pcm_uframes_t  period_size;
@@ -55,26 +73,96 @@ static const char        *channel_name[] = {
   "Rear Left" ,
   "Rear Right" ,
   "Center" ,
-  "LFE" 
+  "LFE",
+  "Side Left",
+  "Side Right",
+  "8",
+  "9",
+  "10",
+  "11",
+  "12",
+  "13",
+  "14",
+  "15",
+  "16" 
 };
 
-static void generate_sine(signed short *samples, int channel, int count, double *_phase) {
+static const int	channels4[] = {
+  0,
+  1,
+  3,
+  2
+};
+static const int	channels6[] = {
+  0,
+  4,
+  1,
+  3,
+  2,
+  5
+}; 
+static const int	channels8[] = {
+  0,
+  4,
+  1,
+  7,
+  3,
+  2,
+  6,
+  5
+}; 
+
+static void generate_sine(uint8_t *frames, int channel, int count, double *_phase) {
   double phase = *_phase;
   double max_phase = 1.0 / freq;
   double step = 1.0 / (double)rate;
   double res;
-  int    chn, ires;
+  int    chn;
+  int32_t  ires;
+  int8_t *samp8 = (int8_t*) frames;
+  int16_t *samp16 = (int16_t*) frames;
+  int32_t *samp32 = (int32_t*) frames;
+  int sample_size_bits = snd_pcm_format_width(format); 
 
   while (count-- > 0) {
-    res = sin((phase * 2 * M_PI) / max_phase - M_PI) * 32767;
+    //res = sin((phase * 2 * M_PI) / max_phase - M_PI) * 32767;
+    //res = sin((phase * 2 * M_PI) / max_phase - M_PI) * 32767;
+    res = (sin((phase * 2 * M_PI) / max_phase - M_PI)) * 0x03fffffff; /* Don't use MAX volume */
+    //if (res > 0) res = 10000;
+    //if (res < 0) res = -10000;
+
     /* printf("%e\n",res); */
     ires = res;
+    //ires = ((16 - (count & 0xf)) <<24);
+    //ires = 0;
 
     for(chn=0;chn<channels;chn++) {
-      if (chn==channel)
-	*samples++ = ires;
-      else
-	*samples++ = 0;
+      if (sample_size_bits == 8) {
+        if (chn==channel) {
+	  *samp8++ = ires >> 24;
+	  //*samp8++ = 0x12;
+        } else {
+	  *samp8++ = 0;
+        }
+      } else if (sample_size_bits == 16) {
+        if (chn==channel) {
+	  *samp16++ = ires >>16;
+	  //*samp16++ = 0x1234;
+        } else {
+	  //*samp16++ = (ires >>16)+1;
+	  *samp16++ = 0;
+        }
+      } else if (sample_size_bits == 32) {
+        if (chn==channel) {
+	  *samp32++ = ires;
+	  //*samp32++ = 0xF2345678;
+	//printf("res=%lf, ires=%d 0x%x, samp32=0x%x\n",res,ires, ires, samp32[-1]);
+        } else {
+	  //*samp32++ = ires+0x10000;
+	  //*samp32++ = ires;
+	  *samp32++ = 0;
+        }
+      }
     }
 
     phase += step;
@@ -85,13 +173,52 @@ static void generate_sine(signed short *samples, int channel, int count, double
   *_phase = phase;
 }
 
-/* FIXME: Implement, because it is a better test than sine wave
- * because we can tell where pink noise is coming from more easily that a sine wave
+/* Pink noise is a better test than sine wave because we can tell
+ * where pink noise is coming from more easily that a sine wave.
  */
-#if 0
-static void generate_pink_noise( snd_pcm_uframes_t offset, int count, double *_phase) {
+
+
+static void generate_pink_noise( uint8_t *frames, int channel, int count) {
+  double   res;
+  int      chn;
+  int32_t  ires;
+  int8_t  *samp8 = (int8_t*) frames;
+  int16_t *samp16 = (int16_t*) frames;
+  int32_t *samp32 = (int32_t*) frames;
+  int sample_size_bits = snd_pcm_format_width(format); 
+
+  while (count-- > 0) {
+    for(chn=0;chn<channels;chn++) {
+      if (sample_size_bits == 8) {
+        if (chn==channel) {
+	  res = generate_pink_noise_sample(&pink) * 0x03fffffff; /* Don't use MAX volume */
+	  ires = res;
+	  *samp8++ = ires >> 24;
+        } else {
+	  *samp8++ = 0;
+        }
+      } else if (sample_size_bits == 16) {
+        if (chn==channel) {
+	  res = generate_pink_noise_sample(&pink) * 0x03fffffff; /* Don't use MAX volume */
+	  ires = res;
+	  *samp16++ = ires >>16;
+        } else {
+	  *samp16++ = 0;
+        }
+      } else if (sample_size_bits == 32) {
+        if (chn==channel) {
+	  res = generate_pink_noise_sample(&pink) * 0x03fffffff; /* Don't use MAX volume */
+	  ires = res;
+	  *samp32++ = ires;
+        } else {
+	  *samp32++ = 0;
+        }
+      }
+    }
+
+  }
+     
 }
-#endif
 
 static int set_hwparams(snd_pcm_t *handle, snd_pcm_hw_params_t *params, snd_pcm_access_t access) {
   unsigned int rrate;
@@ -156,13 +283,18 @@ static int set_hwparams(snd_pcm_t *handle, snd_pcm_hw_params_t *params, snd_pcm_
   err = snd_pcm_hw_params_get_period_size_max(params, &period_size_max,&dir);
   printf("Buffer size range from %lu to %lu\n",buffer_size_min, buffer_size_max);
   printf("Period size range from %lu to %lu\n",period_size_min, period_size_max);
+  printf("Periods = %d\n", PERIODS);
   printf("Buffer time size %lu\n",buffer_time_to_size);
 
   buffer_size = buffer_time_to_size;
+  //buffer_size=8096;
+  buffer_size=15052;
   if (buffer_size_max < buffer_size) buffer_size = buffer_size_max;
   if (buffer_size_min > buffer_size) buffer_size = buffer_size_min;
-  period_size=buffer_size/8;
-  buffer_size = period_size*8;
+  //buffer_size=0x800;
+  period_size = buffer_size/PERIODS;
+  buffer_size = period_size*PERIODS;
+  //period_size = 510;
   printf("To choose buffer_size = %lu\n",buffer_size);
   printf("To choose period_size = %lu\n",period_size);
   dir=0;
@@ -210,14 +342,14 @@ static int set_swparams(snd_pcm_t *handle, snd_pcm_sw_params_t *swparams) {
     return err;
   }
 
-  /* start the transfer when a period is full */
-  err = snd_pcm_sw_params_set_start_threshold(handle, swparams, period_size);
+  /* start the transfer when a buffer is full */
+  err = snd_pcm_sw_params_set_start_threshold(handle, swparams, buffer_size);
   if (err < 0) {
     printf("Unable to set start threshold mode for playback: %s\n", snd_strerror(err));
     return err;
   }
 
-  /* allow the transfer when at least period_size samples can be processed */
+  /* allow the transfer when at least period_size frames can be processed */
   err = snd_pcm_sw_params_set_avail_min(handle, swparams, period_size);
   if (err < 0) {
     printf("Unable to set avail min for playback: %s\n", snd_strerror(err));
@@ -273,15 +405,19 @@ static int xrun_recovery(snd_pcm_t *handle, int err) {
  *   Transfer method - write only
  */
 
-static int write_loop(snd_pcm_t *handle, int channel, int periods, signed short *samples) {
+static int write_loop(snd_pcm_t *handle, int channel, int periods, uint8_t *frames) {
   double phase = 0;
-  signed short *ptr;
+  uint8_t *ptr;
   int    err, cptr, n;
+  int    bytes_per_frame=snd_pcm_frames_to_bytes(handle, 1);
 
   for(n = 0; n < periods; n++) {
-
-    generate_sine(samples, channel, period_size, &phase);
-    ptr = samples;
+    if(test_type==1)
+      generate_pink_noise(frames, channel, period_size);
+    else
+      generate_sine(frames, channel, period_size, &phase);
+      
+    ptr = frames;
     cptr = period_size;
 
     while (cptr > 0) {
@@ -292,14 +428,15 @@ static int write_loop(snd_pcm_t *handle, int channel, int periods, signed short
         continue;
 
       if (err < 0) {
+        printf("Write error: %d,%s\n", err, snd_strerror(err));
         if (xrun_recovery(handle, err) < 0) {
-          printf("Write error: %s\n", snd_strerror(err));
+          printf("xrun_recovery failed: %d,%s\n", err, snd_strerror(err));
 	  return -1;
         }
         break;	/* skip one period */
       }
 
-      ptr += (err * channels);
+      ptr += (err * bytes_per_frame);
       cptr -= err;
     }
   }
@@ -307,7 +444,9 @@ static int write_loop(snd_pcm_t *handle, int channel, int periods, signed short
   return 0;
 }
 
-static void help(void) {
+static void help(void)
+{
+  int k;
 
   printf(
       "Usage: speaker-test [OPTION]... \n"
@@ -316,11 +455,13 @@ static void help(void) {
       "-r,--rate	stream rate in Hz\n"
       "-c,--channels	count of channels in stream\n"
       "-f,--frequency	sine wave frequency in Hz\n"
+      "-F,--format	sample format\n"
       "-b,--buffer	ring buffer size in us\n"
       "-p,--period	period size in us\n"
+      "-t,--test	1=use pink noise, 2=use sine wave\n"
       "-s,--speaker	single speaker test. Values 1=Left or 2=right\n"
       "\n");
-#if 0
+#if 1
   printf("Recognized sample formats are:");
   for (k = 0; k < SND_PCM_FORMAT_LAST; ++k) {
     const char *s = snd_pcm_format_name(k);
@@ -338,17 +479,22 @@ int main(int argc, char *argv[]) {
   int                   err, morehelp;
   snd_pcm_hw_params_t  *hwparams;
   snd_pcm_sw_params_t  *swparams;
-  signed short         *samples;
+  uint8_t              *frames;
   int                   chn;
+  double		time1,time2,time3;
+  struct   timeval	tv1,tv2;
+
   struct option         long_option[] = {
     {"help",      0, NULL, 'h'},
     {"device",    1, NULL, 'D'},
     {"rate",      1, NULL, 'r'},
     {"channels",  1, NULL, 'c'},
     {"frequency", 1, NULL, 'f'},
+    {"format",    1, NULL, 'F'},
     {"buffer",    1, NULL, 'b'},
     {"period",    1, NULL, 'p'},
-    {"speaker",    1, NULL, 's'},
+    {"test",      1, NULL, 't'},
+    {"speaker",   1, NULL, 's'},
     {NULL,        0, NULL, 0  },
   };
 
@@ -361,7 +507,7 @@ int main(int argc, char *argv[]) {
   while (1) {
     int c;
     
-    if ((c = getopt_long(argc, argv, "hD:r:c:f:b:p:s:", long_option, NULL)) < 0)
+    if ((c = getopt_long(argc, argv, "hD:r:c:f:F:b:p:t:s:", long_option, NULL)) < 0)
       break;
     
     switch (c) {
@@ -371,6 +517,9 @@ int main(int argc, char *argv[]) {
     case 'D':
       device = strdup(optarg);
       break;
+    case 'F':
+      format = snd_pcm_format_value(optarg);
+      break;
     case 'r':
       rate = atoi(optarg);
       rate = rate < 4000 ? 4000 : rate;
@@ -396,6 +545,11 @@ int main(int argc, char *argv[]) {
       period_time = period_time < 1000 ? 1000 : period_time;
       period_time = period_time > 1000000 ? 1000000 : period_time;
       break;
+    case 't':
+      test_type = atoi(optarg);
+      test_type = test_type < 1 ? 1 : test_type;
+      test_type = test_type > 2 ? 2 : test_type;
+      break;
     case 's':
       speaker = atoi(optarg);
       speaker = speaker < 1 ? 0 : speaker;
@@ -425,44 +579,79 @@ int main(int argc, char *argv[]) {
 
   printf("Playback device is %s\n", device);
   printf("Stream parameters are %iHz, %s, %i channels\n", rate, snd_pcm_format_name(format), channels);
-  printf("Sine wave rate is %.4fHz\n", freq);
+  if(test_type==1)
+    printf("Using 16 octaves of pink noise\n");
+  else
+    printf("Sine wave rate is %.4fHz\n", freq);
 
-  if ((err = snd_pcm_open(&handle, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
-    printf("Playback open error: %s\n", snd_strerror(err));
-    exit(EXIT_FAILURE);
+loop:
+  while ((err = snd_pcm_open(&handle, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
+    printf("Playback open error: %d,%s\n", err,snd_strerror(err));
+    sleep(1);
   }
 
   if ((err = set_hwparams(handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
     printf("Setting of hwparams failed: %s\n", snd_strerror(err));
+    snd_pcm_close(handle);
+    goto loop;
     exit(EXIT_FAILURE);
   }
+  //getchar();
   if ((err = set_swparams(handle, swparams)) < 0) {
     printf("Setting of swparams failed: %s\n", snd_strerror(err));
+    snd_pcm_close(handle);
+    goto loop;
     exit(EXIT_FAILURE);
   }
 
-  samples = malloc((period_size * channels * snd_pcm_format_width(format)) / 8);
-  if (samples == NULL) {
+  frames = malloc((period_size * channels * snd_pcm_format_width(format)) / 8);
+  initialize_pink_noise( &pink, 16);
+  
+  if (frames == NULL) {
     printf("No enough memory\n");
     exit(EXIT_FAILURE);
   }
   if (speaker==0) {
     while (1) {
 
+      gettimeofday(&tv1, NULL);
       for(chn = 0; chn < channels; chn++) {
-        printf("  - %s\n", channel_name[chn]);
+	int channel=chn;
+	if (channels == 4) {
+	    channel=channels4[chn];
+	}
+	if (channels == 6) {
+	    channel=channels6[chn];
+	}
+	if (channels == 8) {
+	    channel=channels8[chn];
+	}
+        printf(" %d - %s\n", channel, channel_name[channel]);
 
-        err = write_loop(handle, chn, ((rate*5)/period_size), samples);
+        err = write_loop(handle, channel, ((rate*3)/period_size), frames);
+        //err = write_loop(handle, 255, ((rate*3)/period_size), frames);
 
         if (err < 0) {
           printf("Transfer failed: %s\n", snd_strerror(err));
-	  exit(EXIT_FAILURE);
+          free(frames);
+          snd_pcm_close(handle);
+	  printf("Pausing\n");
+	  goto loop ;
+	  //pause();
+	  //printf("Done Pausing\n");
+          exit(EXIT_SUCCESS);
+	  goto loop ;
         }
       }
+      gettimeofday(&tv2, NULL);
+      time1 = (double)tv1.tv_sec + ((double)tv1.tv_usec / 1000000.0);
+      time2 = (double)tv2.tv_sec + ((double)tv2.tv_usec / 1000000.0);
+      time3 = time2 - time1;
+      printf("Time per period = %lf\n", time3 );
     }
   } else {
     printf("  - %s\n", channel_name[speaker-1]);
-    err = write_loop(handle, speaker-1, ((rate*5)/period_size), samples);
+    err = write_loop(handle, speaker-1, ((rate*5)/period_size), frames);
 
     if (err < 0) {
       printf("Transfer failed: %s\n", snd_strerror(err));
@@ -470,7 +659,7 @@ int main(int argc, char *argv[]) {
   }
 
 
-  free(samples);
+  free(frames);
   snd_pcm_close(handle);
 
   exit(EXIT_SUCCESS);
